_G.BJ = _G.BJ or {}
BJ._path = ModPath
BJ._data_path = SavePath .. "bj_data.txt"
BJ._data = {}
BJ._knockout_count = 0

function BJ:Save()
	local file = io.open( self._data_path, "w+" )
	if file then
		file:write( json.encode( self._data ) )
		file:close()
	end
end

function BJ:Load()
	local file = io.open( self._data_path, "r" )
	if file then
		self._data = json.decode( file:read("*all") )
		file:close()
	end
end

-- Advanced localization code originally written by TdlQ
Hooks:Add('LocalizationManagerPostInit', 'LocalizationManagerPostInit_BJ', function(loc)
	for _, filename in pairs(file.GetFiles(BJ._path .. 'loc/')) do
		local str = filename:match('^(.*).txt$')
		if str and Idstring(str) and Idstring(str):key() == SystemInfo:language():key() then
			loc:load_localization_file(BJ._path .. 'loc/' .. filename)
			break
		end
	end
	loc:load_localization_file(BJ._path .. 'loc/english.txt', false)
end)

Hooks:Add( "MenuManagerInitialize", "MenuManagerInitialize_BJ", function( menu_manager )
	MenuCallbackHandler.callback_pager_toggle = function(self, item)
		BJ._data.pager = (item:value() == "on" and true or false)
		BJ:Save()
	end
	BJ:Load()
	MenuHelper:LoadFromJsonFile( BJ._path .. "menu.txt", BJ, BJ._data )
end )

BJ:Load()

if RequiredScript == "lib/units/enemies/cop/copdamage" then
	if Network:is_server() then
		local mvec3_dot = mvector3.dot
		function CopDamage:blackjack_check(attacker_unit, i_body)
			local knockout
			local unit_mov = self._unit:movement()
			if unit_mov then
				if unit_mov:cool() then
					local hit_dir = self._unit:position() - attacker_unit:position()
					local from_behind = mvec3_dot(hit_dir, self._unit:rotation():y()) >= 0
					if from_behind then
						if self:is_head(i_body) then
							local peer_unit = managers.network:session():peer_by_unit(attacker_unit)
							if peer_unit then
								local melee_weap = peer_unit:blackmarket_outfit().melee_weapon
								if melee_weap then
									local weap_tweak = tweak_data.blackmarket.melee_weapons[melee_weap]
									if weap_tweak then
										if not weap_tweak.type or weap_tweak.type ~= "knife" then
											local min_damage_effect = weap_tweak.stats.min_damage_effect
											if min_damage_effect and min_damage_effect > 1 or weap_tweak.instant then
												if not knockout then
													attacker_unit:sound():say("f03b_any", true)
													if not BJ._data.pager then
														self._unit:interaction():set_disabled(true)
														if self._unit:unit_data().has_alarm_pager then
															self._unit:unit_data().has_alarm_pager = false
														end
													end
													knockout = true
												end
											end
										end
									end
								end
							end
						end
					end
				end
			end
			return knockout
		end
		local old_melee = CopDamage.damage_melee
		function CopDamage:damage_melee(attack_data, ...)
			local attacker_unit = attack_data.attacker_unit
			if attacker_unit then
				local col_ray = attack_data.col_ray
				if col_ray then
					local i_body = col_ray.body
					if i_body then
						local knockout = self:blackjack_check(attacker_unit, i_body)
						if knockout then
							attack_data.variant = "blackjack"
						end
					end
				end
			end
			return old_melee(self, attack_data, ...)
		end
		local old_sync = CopDamage.sync_damage_melee
		function CopDamage:sync_damage_melee(attacker_unit, damage_percent, damage_effect_percent, i_body, ...)
			if attacker_unit then
				if i_body then
					local body_name = self._unit:body(i_body)
					if body_name then
						local knockout = self:blackjack_check(attacker_unit, body_name)
						if knockout then
							BJ._knockout_count = BJ._knockout_count + 1
							return self:damage_mission({ variant = "blackjack", attacker_unit = attacker_unit })
						end
					end
				end
			end
			return old_sync(self, attacker_unit, ...)
		end
	end
end

if RequiredScript == "lib/managers/statisticsmanager" then
	local old_anyone = StatisticsManager.killed_by_anyone
	function StatisticsManager:killed_by_anyone(data, ...)
		local variant = data.variant
		if variant then
			if variant == "blackjack" then
				return
			end
		end
		return old_anyone(self, data, ...)
	end
	local old_killed = StatisticsManager.killed
	function StatisticsManager:killed(data, ...)
		local variant = data.variant
		if variant then
			if variant == "blackjack" then
				return
			end
		end
		return old_killed(self, data, ...)
	end
end

if RequiredScript == "lib/network/handlers/unitnetworkhandler" then
	local old_send = UnitNetworkHandler.send_statistics
	function UnitNetworkHandler:send_statistics(total_kills, ...)
		if total_kills then
			if total_kills >= BJ._knockout_count then
				local minus_ko = total_kills - BJ._knockout_count
				total_kills = minus_ko
			end
		end
		return old_send(self, total_kills, ...)
	end
end